﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Windows.Devices.Gpio;
namespace GPIO_InputInterrupt
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {
        private GpioPin myLedPin;
        private const int GPIO_LED_PIN = 0;
        private GpioPin myInterrupt;
        private const int GPIO_INT_PIN = 1;
        private GpioChangeCounter myCounter;
        private GpioChangeReader myReader;


#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public MainWindow()
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            InitializeComponent();
            GPIOSetUp();
        }

        public void GPIOSetUp()
        {
            GpioController myGPIOController = GpioController.GetDefault();
            if (myGPIOController == null)
            {
                txtStatus.Text = "GPIO controller NOT found";
            }
            else
            {
                try
                {
                    myLedPin = myGPIOController.OpenPin(GPIO_LED_PIN);
                    myInterrupt = myGPIOController.OpenPin(GPIO_INT_PIN);
                    myCounter = new GpioChangeCounter(myInterrupt);
                    myCounter.Polarity = GpioChangePolarity.Rising;
                    myReader = new GpioChangeReader(myLedPin);
                    myReader.Polarity = GpioChangePolarity.Both;

                }
                catch (Exception ex)
                {
                    txtStatus.Text = "Error!: " + ex.ToString();
                }
                if (myLedPin == null)
                {
                    txtStatus.Text = "GPIO " + GPIO_LED_PIN.ToString() + " not found";

                }
                else if (myInterrupt == null)
                {
                    txtStatus.Text = "GPIO " + GPIO_INT_PIN.ToString() + " not found";
                }
                else
                {
                    try
                    {
                        myLedPin.SetDriveMode(GpioPinDriveMode.Output);
                        myLedPin.Write(GpioPinValue.High);

                        myInterrupt.SetDriveMode(GpioPinDriveMode.Input);
                        myInterrupt.ValueChanged += MyInterrupt_ValueChanged;

                        myCounter.Start();
                        myReader.Start();
                        txtStatus.Text = "GPIO is ready";
                    }
                    catch (Exception ex)
                    {
                        txtStatus.Text = "Error!: " + ex.ToString();
                    }
                }
            }
        }

        private void MyInterrupt_ValueChanged(GpioPin sender, GpioPinValueChangedEventArgs args)
        {
            if (GpioPinEdge.RisingEdge == args.Edge)
            {
                if (myLedPin.Read() == GpioPinValue.High)
                {
                    myLedPin.Write(GpioPinValue.Low);
                }
                else
                {
                    myLedPin.Write(GpioPinValue.High);
                }
            }
        }

        private void btnGetCount_Click(object sender, RoutedEventArgs e)
        {
            txtStatus.Text = "Interrupt Count: " + myCounter.Read().Count.ToString() + " Relative time:  " + myCounter.Read().RelativeTime.ToString();
        }

        private void btnReadData_Click(object sender, RoutedEventArgs e)
        {
            var myRecords = myReader.GetAllItems();

            txtStatus.Text = "";

            foreach (GpioChangeRecord aRecord in myRecords)
            {
                txtStatus.Text += "Time: " + aRecord.RelativeTime.ToString() + " Edge Type: " + aRecord.Edge.ToString() + "\n";
            }

        }
    }
}
