using System;
using Windows.Devices.SerialCommunication;
using Windows.Devices.Enumeration;
using Windows.Storage.Streams;



namespace Annabooks.SparkFunSerLCD
{
    public class SparkFunSerLCD
    {
        private byte[] LCDCLEAR = new byte[] { 0xFE, 0x01 };
        private byte[] LCDLINE1 = new byte[] { 0xFE, 0x80 };
        private byte[] LCDLINE2 = new byte[] { 0xFE, 0xC0 };
        private byte[] LCDMOVECURR1 = new byte[] { 0xFE, 0x14 };
        private byte[] LCDMOVECURL1 = new byte[] { 0xFE, 0x10 };
        private byte[] LCDSCROLLRIGHT = new byte[] { 0xFE, 0x1C };
        private byte[] LCDSCROLLLEFT = new byte[] { 0xFE, 0x18 };
        private byte[] LCDVISUALON = new byte[] { 0xFE, 0x0C };
        private byte[] LCDVISUALOFF = new byte[] { 0xFE, 0x08 };
        private byte[] LCDCURON = new byte[] { 0xFE, 0x0E };
        private byte[] LCDCUROFF = new byte[] { 0xFE, 0x0C };
        private byte[] LCDBLINKON = new byte[] { 0xFE, 0x0D };
        private byte[] LCDBLINKOFF = new byte[] { 0xFE, 0x0C };

        public SerialDevice UartPort;
        private DataWriter lcdUARTDataWriter;

        private bool lcdReady = false;
        public string connectionError = "";

#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public SparkFunSerLCD(String uart)
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {

            LCDUARTInitialize(uart);

        }

        public async void LCDUARTInitialize(String uartPort)
        {
            try
            {
                string aqs = SerialDevice.GetDeviceSelector(uartPort);
                DeviceInformationCollection deviceInfo = await DeviceInformation.FindAllAsync(aqs);
                UartPort = await SerialDevice.FromIdAsync(deviceInfo[0].Id);
            }
            catch (Exception ex)
            {
                connectionError = ex.Message;
            }
            if (UartPort != null)
            {
                UartPort.BaudRate = 9600;
                UartPort.DataBits = 8;
                UartPort.Parity = SerialParity.None;
                UartPort.StopBits = SerialStopBitCount.One;
                UartPort.Handshake = SerialHandshake.None;

                lcdUARTDataWriter = new DataWriter(UartPort.OutputStream);
                lcdUARTDataWriter.WriteBytes(LCDCLEAR);
                await lcdUARTDataWriter.StoreAsync();
                lcdUARTDataWriter.WriteBytes(LCDCUROFF);
                await lcdUARTDataWriter.StoreAsync();
                lcdReady = true;
            }

        }

        public async void ClearLCD()
        {
            lcdUARTDataWriter.WriteBytes(LCDCLEAR);
            await lcdUARTDataWriter.StoreAsync();
        }

        public async void LCDCursorOn()
        {
            lcdUARTDataWriter.WriteBytes(LCDCURON);
            await lcdUARTDataWriter.StoreAsync();
        }

        public async void LCDCursorOff()
        {

            lcdUARTDataWriter.WriteBytes(LCDCUROFF);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDBlinkOn()
        {

            lcdUARTDataWriter.WriteBytes(LCDBLINKON);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDBlinkOff()
        {

            lcdUARTDataWriter.WriteBytes(LCDBLINKOFF);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDOn()
        {

            lcdUARTDataWriter.WriteBytes(LCDVISUALON);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDOff()
        {

            lcdUARTDataWriter.WriteBytes(LCDVISUALOFF);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDScrollRight()
        {

            lcdUARTDataWriter.WriteBytes(LCDSCROLLRIGHT);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDScrollLeft()
        {

            lcdUARTDataWriter.WriteBytes(LCDSCROLLLEFT);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDMoveCursorRight1()
        {

            lcdUARTDataWriter.WriteBytes(LCDMOVECURR1);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDMoveCursotLeft1()
        {

            lcdUARTDataWriter.WriteBytes(LCDMOVECURL1);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDWriteString(string outputString)
        {

            lcdUARTDataWriter.WriteString(outputString);
            await lcdUARTDataWriter.StoreAsync();
        }

        public async void LCDWriteStringLine1(string outputString)
        {

            lcdUARTDataWriter.WriteBytes(LCDLINE1);
            await lcdUARTDataWriter.StoreAsync();
            lcdUARTDataWriter.WriteString(outputString);
            await lcdUARTDataWriter.StoreAsync();

        }

        public async void LCDWriteStringLine2(string outputString)
        {

            lcdUARTDataWriter.WriteBytes(LCDLINE2);
            await lcdUARTDataWriter.StoreAsync();
            lcdUARTDataWriter.WriteString(outputString);
            await lcdUARTDataWriter.StoreAsync();
        }

        public bool LCDstatus()
        {
            return lcdReady;
        }

    }
}
