﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Windows.Devices.Enumeration;
using Windows.Devices.Spi;
using System.Threading;


namespace SPIPressure
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        private SpiDevice myPressureSensor;
        private const byte SPI_CHIP_SELECT_LINE = 0;


#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public MainWindow()
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            InitializeComponent();
            SPISetup();

        }

        private async void SPISetup()
        {

            try
            {
                SpiConnectionSettings spiSettings = new SpiConnectionSettings(SPI_CHIP_SELECT_LINE);
                spiSettings.ClockFrequency = 8000000;
                spiSettings.Mode = SpiMode.Mode0;

                string aqs = SpiDevice.GetDeviceSelector("SPI2");
                DeviceInformationCollection spiDeviceInfo = await DeviceInformation.FindAllAsync(aqs);
                myPressureSensor = await SpiDevice.FromIdAsync(spiDeviceInfo[0].Id, spiSettings);
                if (myPressureSensor != null)
                {
                    txtStatus.Text = "Pressure sensor ready";
                }
                else
                {
                    txtStatus.Text = "Pressure NOT sensor ready";
                }
            }
            catch (Exception ex)
            {
                txtStatus.Text = ex.Message;
            }
        }

        private void btnGetPressure_Click(object sender, RoutedEventArgs e)
        {
            byte[] coeffData = { 0x88, 0x0, 0x8A, 0x0, 0x8C, 0x0, 0x8E, 0x0, 0x90, 0x0, 0x92, 0x0, 0x94, 0x0, 0x96, 0x0, 0x0 };
            byte[] readCOEFF = new byte[17];

            byte[] starPTconversion = { 0x24, 0x0 };
            byte[] readPTConversionStart = new byte[2];
            byte[] getPT = { 0x80, 0x0, 0x82, 0x0, 0x84, 0x0, 0x86, 0x0, 0x0 };
            byte[] readPT = new byte[9];

            //Read from the SPI port
            myPressureSensor.TransferFullDuplex(coeffData, readCOEFF);
            myPressureSensor.TransferFullDuplex(starPTconversion, readPTConversionStart); // data is always zero
            Thread.Sleep(50);
            myPressureSensor.TransferFullDuplex(getPT, readPT);

            var coefA = readCOEFF[1] << 8 | readCOEFF[3];
            var coefB1 = readCOEFF[5] << 8 | readCOEFF[7];
            var coefB2 = readCOEFF[9] << 8 | readCOEFF[11];
            var coefC12 = readCOEFF[13] << 8 | readCOEFF[15];

            var Padc = (readPT[1] << 8 | readPT[3]) >> 6;
            var Tadc = (readPT[5] << 8 | readPT[7]) >> 6;

            //convert coeffs
            bool coefAneg = false;
            if ((coefA & 0x8000) == 0x8000)
            {
                coefA = (coefA ^ 0xFFFF) + 1; //twos complimnet
                coefAneg = true;
            }

            var coefAInt = coefA >> 3;
            var coefAfrac = coefA & 0x7;
            if (coefAneg)
            {
                coefAInt = -coefAInt;
                coefAfrac = -coefAfrac;
            }
            double a0 = (double)coefAInt + (double)coefAfrac / 8;

            bool coefB1neg = false;
            if ((coefB1 & 0x8000) == 0x8000)
            {
                coefB1 = (coefB1 ^ 0xFFFF) + 1;
                coefB1neg = true;
            }

            var coefB1Int = coefB1 >> 13;
            var coefB1Frac = coefB1 & 0x1FFF;
            if (coefB1neg)
            {
                coefB1Int = -coefB1Int;
                coefB1Frac = -coefB1Frac;
            }
            double b1 = (double)coefB1Int + (double)coefB1Frac / 8192;

            bool coefB2neg = false;
            if ((coefB2 & 0x8000) == 0x8000)
            {
                coefB2 = (coefB2 ^ 0xFFFF) + 1;
                coefB2neg = true;
            }

            var coefB2Int = coefB2 >> 14;
            var coefB2Frac = coefB2 & 0x3FFF;
            if (coefB2neg)
            {
                coefB2Int = -coefB2Int;
                coefB2Frac = -coefB2Frac;
            }
            double b2 = (double)coefB2Int + (double)coefB2Frac / 16384;

            bool coefC12neg = false;
            if ((coefC12 & 0x8000) == 0x8000)
            {
                coefC12 = (coefC12 ^ 0xFFFF) + 1;
                coefC12neg = true;
            }

            var coefC12Frac = coefC12 >> 2;
            if (coefC12neg)
            {
                coefC12Frac = -coefC12Frac;
            }
            double c12 = (double)coefC12Frac / 4194303;

            //Final calculations
            double Pcomp = a0 + ((b1 + (c12 * Tadc)) * Padc) + (b2 * Tadc);
            double PressureResult = (Pcomp * 65 / 1023) + 50;
            txtPressure.Text = Math.Round(PressureResult, 2).ToString();

            double PressureinHg = PressureResult * .02953 / 0.1;
            txtPressureinHg.Text = Math.Round(PressureinHg, 2).ToString();

        }
    }
}
