﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using Annabooks.MPL115A1;
using Annabooks.SHTC3;
using System.Threading;
using Microsoft.Azure.Devices.Provisioning.Client;
using Microsoft.Azure.Devices.Provisioning.Client.Transport;
using Microsoft.Azure.Devices.Shared;
using Microsoft.Azure.Devices.Client;
using System.Text.Json;


namespace Weather1
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow : Window
    {

        private MPL115A1 myPressureSensor;
        private SHTC3 myTempHumidity;
        private string IOTHUB_DEVICE_DPS_ENDPOINT = "global.azure-devices-provisioning.net";
        private string IOTHUB_DEVICE_DPS_ID_SCOPE = "0ne006246AF"; //Application Device Connectivity
        private string IOTHUB_DEVICE_DPS_DEVICE_ID = "WeatherStation1";
        private string IOTHUB_DEVICE_DPS_DEVICE_KEY = "CoILb3fMr4wULMDb4XqAdGPy8hwjffbiY8EwAh5iRWA="; //Generated via Azure CLI
        private const string ModelId = "dtmi:weather:WeatherStation_5yd;1"; //Comes from Template Identity
        private DeviceClient deviceClient;

#pragma warning disable CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        public MainWindow()
#pragma warning restore CS8618 // Non-nullable field must contain a non-null value when exiting constructor. Consider declaring as nullable.
        {
            InitializeComponent();
            Thread setupSensors = new Thread(new ThreadStart(SensorSetup));
            setupSensors.Start();
        }


        private async void SensorSetup()

        {
            //Get an instance of the PressureSensor
            myPressureSensor = new MPL115A1("SPI2", 8000000, 0, false);
            Thread.Sleep(1000);
            if (myPressureSensor.MPL115A1Status())
            {
                Application.Current.Dispatcher.Invoke((Action)(() => txtStatus.Content = "Preasure Sensor is Ready"));
            }
            else
            {
                Application.Current.Dispatcher.Invoke((Action)(() => txtStatus.Content = "Preasure Sensor is NOT Ready: " + myPressureSensor.connectionError));
            }

            //Get an instance of the Temperature / Humidity Sensor
            myTempHumidity = new SHTC3("I2C1", false);
            Thread.Sleep(1100);
            if (myTempHumidity.HTU21DStatus())
            {
                Application.Current.Dispatcher.Invoke((Action)(() => txtStatus.Content += " and Humidity Sensor is Ready"));
            }
            else
            {
                Application.Current.Dispatcher.Invoke((Action)(() => txtStatus.Content += " and Humidity Sensor is NOT Ready: " + myTempHumidity.connectionError));
            }

            //PnP Connect to the Azure IoT Central Application
            //Provision the Device assigned to the Weather Station template using the connection information from the app
            SecurityProvider symmetricKeyProvider = new SecurityProviderSymmetricKey(IOTHUB_DEVICE_DPS_DEVICE_ID, IOTHUB_DEVICE_DPS_DEVICE_KEY, null);
            ProvisioningTransportHandler mqttTransportHandler = new ProvisioningTransportHandlerMqtt();
            ProvisioningDeviceClient pdc = ProvisioningDeviceClient.Create(IOTHUB_DEVICE_DPS_ENDPOINT, IOTHUB_DEVICE_DPS_ID_SCOPE, symmetricKeyProvider, mqttTransportHandler);
            var pnpPayload = new ProvisioningRegistrationAdditionalData
            {
                JsonData = $"{{ \"modelId\": \"{ModelId}\" }}",
            };

            DeviceRegistrationResult dpsRegistrationResult = await pdc.RegisterAsync(pnpPayload);

            var authMethod = new DeviceAuthenticationWithRegistrySymmetricKey(dpsRegistrationResult.DeviceId, IOTHUB_DEVICE_DPS_DEVICE_KEY);

            var options = new ClientOptions
            {
                ModelId = ModelId,
            };
            deviceClient = DeviceClient.Create(dpsRegistrationResult.AssignedHub, authMethod, TransportType.Mqtt, options);


            while (true)
            {
                double _temprature = myTempHumidity.ReadTempC();
                double _humidity = myTempHumidity.ReadHumidity();
                double _pressure = myPressureSensor.ReadPressure_kPa();

                Application.Current.Dispatcher.Invoke((Action)(() => txtTemp.Text = _temprature.ToString()));
                Application.Current.Dispatcher.Invoke((Action)(() => txtHumidity.Text = _humidity.ToString()));
                Application.Current.Dispatcher.Invoke((Action)(() => txtPressure.Text = _pressure.ToString()));

                //Create a Json message
                //The following will group each telemetry item into a single message. The Telemetry names must match the telemetry names set up in the template.
                var weatherTelemetry = new
                {
                    Temperature = _temprature,
                    Humidity = _humidity,
                    BarometricPressure = _pressure,
                };

                var telemetryMessageString = JsonSerializer.Serialize(weatherTelemetry);
                var telemetryMessage = new Message(Encoding.ASCII.GetBytes(telemetryMessageString));

                await deviceClient.SendEventAsync(telemetryMessage);

                Thread.Sleep(5000);
            }
        }

    }
}
